@@ -22,10 +22,11 @@
 #include <string.h>
 #include <errno.h>
 
-#include "gdk-pixbuf-private.h"
-
 #include <jasper/jasper.h>
 
+#include <glib/gi18n-lib.h>
+#include "gdk-pixbuf-io.h"
+
 G_MODULE_EXPORT void fill_vtable (GdkPixbufModule * module);
 G_MODULE_EXPORT void fill_info (GdkPixbufFormat * info);
 
